importClass(Packages.com.alarmpoint.integrationagent.http.HttpClientWrapper);
importClass(Packages.com.alarmpoint.integrationagent.http.DefaultX509TrustManager);
importClass(Packages.com.alarmpoint.integrationagent.soap.SOAPMessage);
importClass(Packages.com.alarmpoint.integrationagent.soap.SOAPRequest);
importClass(Packages.com.alarmpoint.integrationagent.soap.exception.SOAPRequestException);
importClass(Packages.com.alarmpoint.integrationagent.util.EventDeduplicator);
importClass(Packages.javax.net.ssl.SSLContext);
importClass(Packages.javax.net.ssl.TrustManager);
importClass(Packages.org.apache.commons.lang.StringEscapeUtils);
importClass(Packages.org.apache.commons.codec.binary.Base64);
importClass(Packages.org.apache.http.conn.ssl.SSLSocketFactory);

// Add trim functionality to the JavaScript String prototype
String.prototype.trim = function() {
	return this.replace(/^\s+|\s+$/g,"");
}
String.prototype.ltrim = function() {
	return this.replace(/^\s+/,"");
}
String.prototype.rtrim = function() {
	return this.replace(/\s+$/,"");
}

String.prototype.endsWith = function(pattern) {
    var d = this.length - pattern.length;
    return d >= 0 && this.lastIndexOf(pattern) === d;
}

String.prototype.startsWith = function(str){
    return (this.indexOf(str) === 0);
}

var WSUtil = BaseClass.extend({

  POST: "POST",
  GET: "GET",
  PUT: "PUT",
  HEAD: "HEAD",

  // Use SSL but trust any certificate (including self-signed ones).
  // This SHOULD NOT be used for productive systems due to security reasons, 
  // unless it is a conscious decision and you are perfectly aware of security 
  // implications of accepting self-signed certificates  
  ACCEPT_ANY_CERTIFICATE: true,
  
  // File/Path for the Java Keystore containing the trusted certificate chain
  KEY_STORE: "jre/lib/security/cacerts",
  
  // Password for the Java Keystore containing the trusted certificate chain
  KEY_STORE_PASSWORD: "changeit",
  
  // SSL Socket Factory used for SSL communication
  SSL_SOCKET_FACTORY: null,

  /**
   * Constructor
   */
  init: function()
  {
    this.log = new Logger();
  },

  /**
   * Sends the request to the SOAP endpoint and returns the response
   * @param url EndPoint for the webservice request
   * @param msg SOAPMessage instance containing the SOAPEnvelope to post
   * @param headers array of SOAP Headers to be added to the SOAP Request
   * @return SOAP Response Body
   */
  sendReceive: function(url, msg, headers)
  {
    // These variables are used by E4X to access nodes prefixed with a namespace. These namespaces are defined
    // in the SOAP Response that is parsed by E4X
    var soap = new Namespace("http://schemas.xmlsoap.org/soap/envelope/");

    var request;
    if (url.toLowerCase().startsWith("https"))
    {
      if (this.ACCEPT_ANY_CERTIFICATE)
      {
        request = new SOAPRequest(url, msg.getAction(), this.ACCEPT_ANY_CERTIFICATE);
        this.log.debug("SSL enabled to accept ANY certificate");
      }
      else
      {
        request = new SOAPRequest(url, msg.getAction(), this.getSSLSocketFactory());
        this.log.debug("SSL enabled using Trusted CA Certificate");
      }
    }
    else
    {
      request = new SOAPRequest(url, msg.getAction(), false);
    }
    
    if (headers != null)
    {
      for (var it = headers.keySet().iterator(); it.hasNext();)
      {
        var key = it.next();
        var value = headers.get(key);
        this.log.debug(key + "=" + value);
        request.getSoapHeaders().put(key, value);
      }
    }
    return new XML(request.post(msg.getSoapEnvelope(msg.getRequest()))).soap::Body;
  },
  
  
  /**
   * Sends the request to the REST endpoint and returns the response
   * @param protocol HTTP or HTTPS
   * @param server FQDN or IP of the server hosting the REST service
   * @param port integer representing the port the REST service is exposed on
   * @param rootPath path for the REST service
   * @param username name of the user making the request
   * @param password clear text password (that will be encrypted prior to sending to the REST service) 
   * @param method String representing the request method (GET, POST, PUT etc)
   * @param request body of the request to send to the REST Service
   * @param headers array of Request Headers to be added to the Request
   * @return Response Body
   */
  restSendReceive: function(protocol, server, port, rootPath, username, password, method, request, headers)
  {
    var client;
    if (protocol.toLowerCase().startsWith("https"))
    {
      if (this.ACCEPT_ANY_CERTIFICATE)
      {
        client = new HttpClientWrapper(username, password, server, port, rootPath, true);
        this.log.debug("SSL enabled to accept ANY certificate");
      }
      else
      {
        client = new HttpClientWrapper(username, password, server, port, rootPath, this.getSSLSocketFactory());
        this.log.debug("SSL enabled using Trusted CA Certificate");
      }
    }
    else
    {
      client = new HttpClientWrapper(username, password, server, port, rootPath);
    }
    
    var response = null;
    var responseStr = "";
    if (this.POST.equalsIgnoreCase(method))
    {
      response = client.executeHttpPost(request);
      responseStr = client.getResponseAsString(response);
    }
    else if (this.GET.equalsIgnoreCase(method))
    {
      response = client.executeHttpGet();
      responseStr = client.getResponseAsString(response);
    }
    else if (this.PUT.equalsIgnoreCase(method))
    {
      response = client.executeHttpPut(request);
      responseStr = client.getResponseAsString(response);
    }
    
    //If there is a response body we are probably more interested in that so return it.
    if (responseStr != "")
    {
      response = responseStr;
    }
    return response;
  },

  /**
   * Encodes the given string using the base64 algorithm
   * @param msg String to encode
   * @return a base64 encrypted String
   */
  base64Encode: function(msg)
  {
    var encoder = new Base64();
    return new java.lang.String(encoder.encodeBase64(new java.lang.String(msg).getBytes("UTF8")), "UTF8");
  },
  
  /**
   * The default implementation of SSL between the IA and REST/SOAP endpoints
   * will allow for self-signed certificates. To implement more secure SSL
   * communication, an instance of an X509TrustManager should be created that
   * points to the keystore containing a signed certificate with the appropriate
   * credentials.
   *
   * The trusted certificate MUST be imported into the Java Keystore used by
   * the Integration Agent (conf/.keystore) using the credentials defined above.
   * In order for the trust chain to be enabled, the ACCEPT_ANY_CERTIFICATE
   * property above MUST be set to false                                         
   */
  getSSLSocketFactory: function()
  {
    if (this.SSL_SOCKET_FACTORY == null)
    {
      var sslContext = SSLContext.getInstance("TLS");
      var trustManagers = java.lang.reflect.Array.newInstance(TrustManager, 1);
      trustManagers[0] = new DefaultX509TrustManager(this.KEY_STORE, this.KEY_STORE_PASSWORD); 
      sslContext.init(null, trustManagers , null);
      this.SSL_SOCKET_FACTORY = new SSLSocketFactory(sslContext, SSLSocketFactory.STRICT_HOSTNAME_VERIFIER);
    }
    return this.SSL_SOCKET_FACTORY;
  }
  
  
});
